package com.loafle.overflow.container.server.handler;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCServerCodec;
import com.loafle.commons.rpc.protocol.RPCServerRequestCodec;
import com.loafle.commons.rpc.registry.RPCInvoker;
import com.loafle.commons.server.socket.handler.codec.SocketFrame;
import com.loafle.commons.server.socket.handler.codec.TextSocketFrame;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;

/**
 * RPCDecoder
 */
@Component
public class RPCServerHandler extends SimpleChannelInboundHandler<SocketFrame> {
  private static final Logger logger = LoggerFactory.getLogger(RPCServerHandler.class);

  @Autowired
  private RPCServerCodec serverCodec;
  @Autowired
  private RPCInvoker rpcInvoker;

  public RPCServerHandler() {
  }

  @Override
  protected void channelRead0(ChannelHandlerContext ctx, SocketFrame frame) throws Exception {
    if (frame instanceof TextSocketFrame) {
      byte[] requestBytes = null;
      RPCServerRequestCodec requestCodec = null;
      Object reply = null;
      RPCException error = null;
      try {
        requestBytes = new byte[frame.content().readableBytes()];
        frame.content().readBytes(requestBytes);

        requestCodec = this.serverCodec.request(requestBytes);
        reply = this.rpcInvoker.invoke(requestCodec);
        
      } catch (RPCException e) {
        logger.debug("Error", e);
        error = e;
      }
      if (!requestCodec.hasResponse()) {
        return;
      }
      byte[] responseBytes = requestCodec.response(reply, error);
      ctx.channel().writeAndFlush(responseBytes);
    } else {
        throw new UnsupportedOperationException(String.format("unsupported frame type: %s", frame.getClass().getName()));
    }
  }
}